"""Organization mutations."""

import json
from typing import Optional

from typeguard import typechecked

from kili.core.graphql.graphql_client import GraphQLClient
from kili.entrypoints.base import BaseOperationEntrypointMixin
from kili.utils.logcontext import for_all_methods, log_call

from .queries import GQL_CREATE_ORGANIZATION, GQL_UPDATE_PROPERTIES_IN_ORGANIZATION


@for_all_methods(log_call, exclude=["__init__"])
class MutationsOrganization(BaseOperationEntrypointMixin):
    """Set of Organization mutations."""

    graphql_client: GraphQLClient

    # pylint: disable=too-many-arguments
    @typechecked
    def create_organization(self, name: str, address: str, zip_code: str, city: str, country: str):
        """Create an organization.

        WARNING: This method is for internal use only.

        Each user must be linked to an organization

        Args:
            name : Name of the organization
            address : Address of the organization
            zip_code : Zip code of the organization
            city : City of the organization
            country : Country of the organization

        Returns:
            A result object which indicates if the mutation was successful,
                or an error message.
        """
        variables = {
            "data": {
                "name": name,
                "address": address,
                "zipCode": zip_code,
                "city": city,
                "country": country,
            }
        }
        result = self.graphql_client.execute(GQL_CREATE_ORGANIZATION, variables)
        return self.format_result("data", result)

    @typechecked
    def update_properties_in_organization(
        self,
        organization_id: str,
        name: Optional[str] = None,
        license: Optional[dict] = None,
    ):  # pylint: disable=redefined-builtin
        """Modify an organization.

        WARNING: This method is for internal use only.

        Args:
            organization_id :
            name :
            license :

        Returns:
            A result object which indicates if the mutation was successful,
                or an error message.
        """
        license_str = None if not license else json.dumps(license)
        variables = {"id": organization_id}
        if name is not None:
            variables["name"] = name
        if license_str is not None:
            variables["license"] = license_str
        result = self.graphql_client.execute(GQL_UPDATE_PROPERTIES_IN_ORGANIZATION, variables)
        return self.format_result("data", result)



"""Common code for the GeoJson exporter."""

import json
from pathlib import Path
from typing import Dict, List

from kili.orm import JobMLTask, JobTool
from kili.services.export.format.base import AbstractExporter
from kili.services.export.tools import is_geotiff_asset_with_lat_lon_coords
from kili.services.types import Job
from kili.utils.labels.geojson import kili_json_response_to_feature_collection
from kili.utils.tqdm import tqdm

from ...exceptions import NotCompatibleInputType, NotCompatibleOptions


class GeoJsonExporter(AbstractExporter):
    """Common code for GeoJson exporter."""

    def _check_arguments_compatibility(self) -> None:
        """Checks if the export label format is compatible with the export options."""
        if self.normalized_coordinates is not None:
            raise NotCompatibleOptions(
                "The GeoJson annotation format can only be exported with lat/long coordinates."
                " Please set `normalized_coordinates` to None."
            )

        if self.split_option != "merged":
            raise NotCompatibleOptions(
                "The current implementation only supports merged annotations."
            )

        if self.single_file:
            raise NotCompatibleOptions(
                "The GeoJson annotation format cannot be exported into a single file."
            )

    def _check_project_compatibility(self) -> None:
        """Checks if the export label format is compatible with the project."""
        if self.project["inputType"] != "IMAGE":
            raise NotCompatibleInputType(
                f"Project with input type '{self.project['inputType']}' not compatible with"
                " GeoJson export format."
            )

    def _is_job_compatible(self, job: Job) -> bool:
        """Check if the export label format is compatible with the job."""
        if "tools" not in job:
            return False

        if job["mlTask"] != JobMLTask.ObjectDetection:
            return False

        compatible_tools = {
            JobTool.Rectangle,
            JobTool.Polygon,
            JobTool.Semantic,
            JobTool.Marker,
            JobTool.Polyline,
        }

        return all(tool in compatible_tools for tool in job["tools"])

    def process_and_save(self, assets: List[Dict], output_filename: Path) -> None:
        self.logger.info("Exporting to GeoJson format")

        labels_folder = self.base_folder / "labels"
        labels_folder.mkdir(parents=True, exist_ok=True)

        geotiff_assets = [
            asset
            for asset in assets
            if is_geotiff_asset_with_lat_lon_coords(asset, self.kili.http_client)
        ]
        if len(geotiff_assets) < len(assets):
            self.logger.warning(
                f"Among {len(assets)} assets, only {len(geotiff_assets)} are geotiff assets and"
                " will be exported."
            )

        for asset in tqdm(geotiff_assets, disable=self.disable_tqdm):
            _process_asset(asset, labels_folder)

        self.create_readme_kili_file(self.export_root_folder)
        self.make_archive(self.export_root_folder, output_filename)

        self.logger.warning(output_filename)


def _process_asset(asset: Dict, labels_folder: Path) -> None:
    geojson_feature_collection = kili_json_response_to_feature_collection(
        asset["latestLabel"]["jsonResponse"]
    )

    filename = f'{asset["externalId"]}.geojson'
    with open(labels_folder / filename, "w", encoding="utf-8") as file:
        json.dump(geojson_feature_collection, file)



"""Service for importing objects into kili."""

from typing import Dict, List, Type, cast

from kili.services.asset_import.image import ImageDataImporter
from kili.services.asset_import.pdf import PdfDataImporter
from kili.services.asset_import.text import TextDataImporter
from kili.services.asset_import.types import AssetLike
from kili.services.asset_import.video import VideoDataImporter
from kili.services.project import get_project_field

from .base import (
    BaseAbstractAssetImporter,
    LoggerParams,
    ProcessingParams,
    ProjectParams,
)

importer_by_type: Dict[str, Type[BaseAbstractAssetImporter]] = {
    "PDF": PdfDataImporter,
    "IMAGE": ImageDataImporter,
    "TEXT": TextDataImporter,
    "VIDEO": VideoDataImporter,
    "VIDEO_LEGACY": VideoDataImporter,
}


def import_assets(  # pylint: disable=too-many-arguments
    kili,
    project_id: str,
    assets: List[Dict],
    raise_error=True,
    disable_tqdm=False,
    verify=True,
):
    """Import the selected assets into the specified project."""
    input_type = get_project_field(kili, project_id, "inputType")

    project_params = ProjectParams(project_id=project_id, input_type=input_type)
    processing_params = ProcessingParams(raise_error=raise_error, verify=verify)
    logger_params = LoggerParams(disable_tqdm=disable_tqdm)
    importer_params = (kili, project_params, processing_params, logger_params)

    if input_type not in importer_by_type:
        raise NotImplementedError(f"There is no imported for the input type: {input_type}")
    asset_importer = importer_by_type[input_type](*importer_params)
    created_asset_ids = asset_importer.import_assets(assets=cast(List[AssetLike], assets))
    return created_asset_ids
